searchData={"items":[{"type":"module","title":"Extep","doc":"A tiny and friendly step runner for Elixir pipelines.\n\nExtep is a simple and dependency-free utility that helps you compose Elixir pipelines using a shared context.\nIt's useful for building multi-step workflows that can gracefully **halt** or **error** along the way.\nExtep is an implementation of the Railway-oriented programming and was inspired by\n[Ecto.Multi](https://hexdocs.pm/ecto/Ecto.Multi.html) and [Sage](https://hexdocs.pm/sage/readme.html).","ref":"Extep.html"},{"type":"module","title":"Example - Extep","doc":"iex> params = %{user_id: 1, plan: \"super-power-plus\"}\n    iex> Extep.new(%{params: params})\n    ...> |> Extep.run(:params, &validate_params/1)\n    ...> |> Extep.run(:user, &fetch_user/1)\n    ...> |> Extep.run(:items, &fetch_items/1)\n    ...> |> Extep.return(&create_subscription/1)\n    {:ok, %{id: 123, object: \"subscription\", user_id: 1, items: [%{code: \"item1\"}, %{code: \"item2\"}]}}","ref":"Extep.html#module-example"},{"type":"type","title":"Extep.context/0","doc":"","ref":"Extep.html#t:context/0"},{"type":"type","title":"Extep.context_checker_fun/0","doc":"","ref":"Extep.html#t:context_checker_fun/0"},{"type":"type","title":"Extep.context_key/0","doc":"","ref":"Extep.html#t:context_key/0"},{"type":"type","title":"Extep.context_mutator_fun/0","doc":"","ref":"Extep.html#t:context_mutator_fun/0"},{"type":"function","title":"Extep.new/0","doc":"Creates a new `Extep` struct with an empty context.","ref":"Extep.html#new/0"},{"type":"function","title":"Examples - Extep.new/0","doc":"iex> Extep.new()\n    %Extep{status: :ok, context: %{}, message: nil}","ref":"Extep.html#new/0-examples"},{"type":"function","title":"Extep.new/1","doc":"Creates a new `Extep` struct with an initial context map.","ref":"Extep.html#new/1"},{"type":"function","title":"Examples - Extep.new/1","doc":"iex> Extep.new(%{foo: \"bar\"})\n    %Extep{status: :ok, context: %{foo: \"bar\"}, message: nil}","ref":"Extep.html#new/1-examples"},{"type":"function","title":"Extep.return/2","doc":"Returns a final result from your pipeline.\n\nIf your `Extep` status is `:ok` and you pass a function, it calls the function\nwith the context and returns whatever it returns. If you pass a context key,\nit fetches the value for that key and returns it in an ok tuple.\n\nIf the pipeline was halted, it returns the content of the `:message` field of the Extep struct.\nIf the pipeline errored, it returns `{:error, message}`.","ref":"Extep.html#return/2"},{"type":"function","title":"Examples - Extep.return/2","doc":"iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(:bar, fn ctx -> {:ok, ctx.foo + 1} end)\n    ...> |> Extep.return(fn ctx -> {:ok, ctx.bar + 2} end)\n    {:ok, 4}\n\n    iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(:bar, fn ctx -> {:ok, ctx.foo + 1} end)\n    ...> |> Extep.run(:baz, fn ctx -> {:ok, ctx.bar + 2} end)\n    ...> |> Extep.return(:bar)\n    {:ok, 2}\n\n    iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(:bar, fn ctx -> {:ok, ctx.foo + 1} end)\n    ...> |> Extep.return(&return_error_tuple/1)\n    {:error, %{return_error_tuple: \"error message\"}}\n\n    iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(:bar, fn ctx -> {:ok, ctx.foo + 1} end)\n    ...> |> Extep.run(:baz, &return_error_tuple/1)\n    ...> |> Extep.return(:bar)\n    {:error, %{return_error_tuple: \"error message\"}}\n\n    iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(:bar, fn ctx -> {:ok, ctx.foo + 1} end)\n    ...> |> Extep.run(:baz, fn _ctx -> {:error, \"error message\"} end)\n    ...> |> Extep.return(:bar)\n    {:error, %{baz: \"error message\"}}\n\n    iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(:bar, fn ctx -> {:ok, ctx.foo + 1} end)\n    ...> |> Extep.run(fn _ctx -> {:error, \"error message\"} end)\n    ...> |> Extep.return(:bar)\n    {:error, %{no_context_key: \"error message\"}}\n\n    iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(:bar, fn ctx -> {:ok, ctx.foo + 1} end)\n    ...> |> Extep.return(fn _ctx -> {:halt, {:ok, \"halt message\"}} end)\n    {:ok, \"halt message\"}\n\n    iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(:bar, fn ctx -> {:ok, ctx.foo + 1} end)\n    ...> |> Extep.return(fn _ctx -> {:halt, {:cancel, \"cancel message\"}} end)\n    {:cancel, \"cancel message\"}\n\n    iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(:bar, fn ctx -> {:ok, ctx.foo + 1} end)\n    ...> |> Extep.run(:baz, fn _ctx -> {:halt, {:ok, \"halt message\"}} end)\n    ...> |> Extep.return(:bar)\n    {:ok, \"halt message\"}","ref":"Extep.html#return/2-examples"},{"type":"function","title":"Extep.run/2","doc":"Runs a checker function on the context without changing it.\n\nIf the function returns `:ok` or `{:ok, \"value\"}`, the pipeline continues.\nIf it returns `{:halt, \"reason\"}` or `{:error, \"reason\"}`, the pipeline stops and\nthe reason is set in the `:message` field of the Extep struct.","ref":"Extep.html#run/2"},{"type":"function","title":"Examples - Extep.run/2","doc":"iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(fn _ctx -> :ok end)\n    %Extep{status: :ok, context: %{foo: 1}, message: nil}\n\n    iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(fn _ctx -> {:ok, \"is valid\"} end)\n    %Extep{status: :ok, context: %{foo: 1}, message: nil}\n\n    iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(&return_error_tuple/1)\n    %Extep{status: :error, context: %{foo: 1}, message: %{return_error_tuple: \"error message\"}}\n\n    iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(fn _ctx -> {:error, \"error message\"} end)\n    %Extep{status: :error, context: %{foo: 1}, message: %{no_context_key: \"error message\"}}\n\n    iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(:foo, fn _ctx -> {:halt, \"halt message\"} end)\n    %Extep{status: :halted, context: %{foo: 1}, message: \"halt message\"}","ref":"Extep.html#run/2-examples"},{"type":"function","title":"Extep.run/3","doc":"Runs a mutator function that updates the context with the result under the given key.\n\nIf the function returns `{:ok, \"value\"}`, \"value\" is set to the given key and the\npipeline continues.\nIf it returns `{:halt, \"reason\"}` or `{:error, \"reason\"}`, the pipeline stops and\nthe reason is set in the `:message` field of the Extep struct.","ref":"Extep.html#run/3"},{"type":"function","title":"Examples - Extep.run/3","doc":"iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(:foo, fn ctx -> {:ok, ctx.foo + 1} end)\n    ...> |> Extep.run(:bar, fn ctx -> {:ok, ctx.foo + 2} end)\n    %Extep{status: :ok, context: %{foo: 2, bar: 4}, message: nil}\n\n    iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(:foo, &return_error_tuple/1)\n    ...> |> Extep.run(:bar, fn ctx -> {:ok, ctx.foo + 2} end)\n    %Extep{status: :error, context: %{foo: 1}, message: %{return_error_tuple: \"error message\"}}\n\n    iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(:foo, fn _ctx -> {:error, \"error message\"} end)\n    ...> |> Extep.run(:bar, fn ctx -> {:ok, ctx.foo + 2} end)\n    %Extep{status: :error, context: %{foo: 1}, message: %{foo: \"error message\"}}\n\n    iex> Extep.new(%{foo: 1})\n    ...> |> Extep.run(:foo, fn _ctx -> {:halt, {:ok, \"halt message\"}} end)\n    ...> |> Extep.run(:bar, fn ctx -> {:ok, ctx.foo + 2} end)\n    %Extep{status: :halted, context: %{foo: 1}, message: {:ok, \"halt message\"}}","ref":"Extep.html#run/3-examples"},{"type":"type","title":"Extep.status/0","doc":"","ref":"Extep.html#t:status/0"},{"type":"type","title":"Extep.t/0","doc":"","ref":"Extep.html#t:t/0"},{"type":"extras","title":"Overview","doc":"# Extep\n\n![**Extep**](https://raw.githubusercontent.com/shore-gmbh/extep/main/Extep.png)\n\n**A tiny and friendly step runner for Elixir pipelines.**\n\nExtep is a simple and dependency-free utility that helps you compose Elixir pipelines using a shared context. It's useful for building multi-step workflows that can gracefully **halt** or **error** along the way. Extep is an implementation of the Railway-oriented programming and was inspired by [Ecto.Multi](https://hexdocs.pm/ecto/Ecto.Multi.html) and [Sage](https://hexdocs.pm/sage/readme.html).","ref":"readme.html"},{"type":"extras","title":"Installation - Overview","doc":"Add `extep` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:extep, \"~> 0.1.0\"}\n  ]\nend\n```\n\nThe full documentation can be found at https://hexdocs.pm/extep.","ref":"readme.html#installation"},{"type":"extras","title":"Basic Usage - Overview","doc":"Each step is a function that either checks something or returns a value to be added to the context.\n\n```elixir\nparams = %{user_id: 1, plan: \"super-power-plus\"}\n\nExtep.new(%{params: params})\n|> Extep.run(:params, &validate_params/1)\n|> Extep.run(:user, &fetch_user/1)\n|> Extep.run(:items, &fetch_items/1)\n|> Extep.return(&create_subscription/1)\n#=> {:ok, %{id: 123, object: \"subscription\", user_id: 1, items: [%{code: \"item1\"}, %{code: \"item2\"}]}}\n\ndef validate_params(%{params: %{user_id: _id}} = ctx), do: {:ok, ctx.params}\n\ndef fetch_user(%{params: %{user_id: id}}), do: {:ok, %{id: id, name: \"Alice\"}}\n\ndef fetch_items(_ctx), do: {:ok, [%{code: \"item1\"}, %{code: \"item2\"}]}\n\ndef create_subscription(%{user: user, items: items}) do\n  {:ok, %{id: 123, object: \"subscription\", user_id: user.id, items: items}}\nend\n```","ref":"readme.html#basic-usage"},{"type":"extras","title":"API - Overview","doc":"### `Extep.new/0`\n\nCreates a new, empty Extep struct:\n\n```elixir\nExtep.new()\n#=> %Extep{status: :ok, context: %{}, message: nil}\n```\n\n### `Extep.new/1`\n\nCreates a new Extep struct with a given context map:\n\n```elixir\nExtep.new(%{foo: \"bar\"})\n#=> %Extep{status: :ok, context: %{foo: \"bar\"}, message: nil}\n```\n\n### `Extep.run/2`\n\nRuns a checker function that must return one of the following:\n\n- `:ok` continues the pipeline;\n- `{:ok, value}` continues the pipeline;\n- `{:halt, reason}` halts pipeline, stores `reason` in `:message`;\n- `{:error, reason}` errors out, stores `reason` in `:message`.\n\n```elixir\nExtep.new(%{foo: 1})\n|> Extep.run(&check_something/1)\n#=> %Extep{status: :ok, context: %{foo: 1}, message: nil}\n```\n\n### `Extep.run/3`\n\nRuns a mutator function and stores the result under the given key. It must return one of the following:\n\n- `{:ok, value}` continues the pipeline, saves `value` under the given key;\n- `{:halt, reason}` halts pipeline, stores `reason` in `:message`;\n- `{:error, reason}` errors out, stores `reason` in `:message`.\n\n```elixir\nExtep.new(%{foo: 1})\n|> Extep.run(:bar, fn ctx -> {:ok, ctx.foo + 1} end)\n#=> %Extep{status: :ok, context: %{foo: 1, bar: 2}, message: nil}\n```\n\n### `Extep.return/2`\n\nReturns a final result from the pipeline:\n\n- With a function:\n\n```elixir\nExtep.new(%{foo: 1})\n|> Extep.run(:bar, fn ctx -> {:ok, ctx.foo + 1} end)\n|> Extep.return(fn ctx -> {:ok, ctx.bar + 2} end)\n#=> {:ok, 4}\n```\n\n- With a context key:\n\n```elixir\nExtep.new(%{foo: 1})\n|> Extep.run(:bar, fn ctx -> {:ok, ctx.foo + 1} end)\n|> Extep.run(:baz, fn ctx -> {:ok, ctx.bar + 2} end)\n|> Extep.return(:bar)\n#=> {:ok, 2}\n```","ref":"readme.html#api"},{"type":"extras","title":"Error Messages - Overview","doc":"If a step fails or halts, Extep stores the reason in the `:message` field. For functions passed to `run/3`, the key is the same as the context key. For anonymous functions in `run/2`, the key is `:no_context_key`.","ref":"readme.html#error-messages"},{"type":"extras","title":"License - Overview","doc":"MIT © [Shore GmbH]","ref":"readme.html#license"},{"type":"extras","title":"License","doc":"MIT License\n\nCopyright (c) [2025] [Shore GmbH]\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}